
###################################################################################################
# MSB_ADJ.py  - Adjustment Utility for 16 bit R2R Ladder DAC -- Tested on Raspberry PI Python 3.4.2
# Author:       Oli
# Date:         01/05/2017
###################################################################################################

#!/usr/bin/python3          # Allows this Python script to run from command line

# Relationship between pins and GPIO signals:

    # Pin 12 : GPIO 18 - CLK (clock)
    # Pin 35 : GPIO 19 - LE (latch enable)
    # Pin 40 : GPIO 21 - DATA
    # Pin 39 : GND

# Import required modules, setup GPIO signals as outputs and initial signal states 

import RPi.GPIO as GPIO     # Imports GPIO module
import time                 # Imports time module
import os                   # Imports os module
GPIO.setwarnings(False)     # Prevents warnings if GPIO 18,19,21 are already used by DAC driver - overidden by this utility
GPIO.setmode(GPIO.BCM)      # Tells the library to refernce using GPIO signal numbers 
GPIO.setup(18, GPIO.OUT)    # Sets up GPIO 18 as output for CLK
GPIO.setup(19, GPIO.OUT)    # Sets up GPIO 19 as output for LE
GPIO.setup(21, GPIO.OUT)    # Sets up GPIO 21 as output for DATA
GPIO.output(19, GPIO.HIGH)  # Sets LE high

# Define variables

bit_number = 0                                          # Defines inital value of bit_number = 0
clock_period = 0.1                                      # Defines clock_period
data_word = (1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0)           # Defines the data_word

# Print headings for table

os.system('clear')                                      # Clears the screen
print ('''
------------------------------------------------------
MSB_ADJ - Adjustment Utility for 16 bit R2R Ladder DAC
------------------------------------------------------

Author: Oli
Date:   01/05/2017
''')                                                    # Prints title, author and date details
print 'Bit',u'\u0009','DATA',u'\u0009','CLK'            # Prints heading for table Bit > DATA > CLK

# Main program loop

for data_value in data_word:                            # Loops over each piece of data listed in data_word 
    
    print ('-------------------')                           # Prints a dotted line
    GPIO.output(18,GPIO.LOW)                                # Sets CLK low
    bit_number = bit_number + 1                             # Adds 1 to the bit_number
    
    if data_value == 1:                                         # Conditions executed if a data_value is high
        
        GPIO.output(21,GPIO.HIGH)                                   # Sets DATA high
        print (bit_number),u'\u0009',u'\u25a0',u'\u0009',u'\u25a1'  # Prints bit_number > a filled square (for high DATA) > an empty square (for low CLK)
        time.sleep(clock_period / 2)                                # Waits for 1/2 CLK period
        GPIO.output(18,GPIO.HIGH)                                   # Sets CLK high - clocks in data
        print u'\u0009',u'\u25a0',u'\u0009',u'\u25a0'               # Prints bit_number > a filled square (for high DATA) > a filled square (for high CLK)
        time.sleep(clock_period / 2)                                # Waits for 1/2 CLK period
        
    if data_value == 0:                                         # Conditions executed if a data_value is low                                           

        GPIO.output(21,GPIO.LOW)                                    # Sets DATA low
        print (bit_number),u'\u0009',u'\u25a1',u'\u0009',u'\u25a1'  # Prints bit_number > an empty square (for low DATA) > an unfilled square (for low CLK)
        time.sleep(clock_period /2 )                                # Waits for 1/2 CLK period
        GPIO.output(18,GPIO.HIGH)                                   # Sets CLK high - clocks in data
        print u'\u0009',u'\u25a1',u'\u0009',u'\u25a0'               # Prints bit_number > an empty square (for low DATA) > a filled square (for high CLK)
        time.sleep(clock_period / 2)                                # Waits for 1/2 CLK period

# Latching in data and printing end of table
    
GPIO.output(18, GPIO.LOW)                               # Set CLK low
GPIO.output(19, GPIO.LOW)                               # Set LE low - latches in data

print ('''-------------------')                           
                                            
Now shut down, switch off the power supply to your your PI and measure Vout from DAC
      
''')                                                    # Prints end of table and final instructions before measurement                         
